/*
 * Copyright (C) 2014-2019 Igor Maznitsa
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.igormaznitsa.zxpoly.tracer;

import com.igormaznitsa.zxpoly.components.ZxPolyModule;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

public class MemoryDialog extends javax.swing.JDialog {

  private final ZxPolyModule module;
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton buttonClose;
  private javax.swing.JButton buttonSet;
  private com.igormaznitsa.zxpoly.tracer.HexValue4Field fieldAddress;
  private com.igormaznitsa.zxpoly.tracer.HexValue2Field fieldValue;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JLabel jLabel2;

  public MemoryDialog(final java.awt.Frame parent, final boolean modal, final ZxPolyModule module) {
    super(parent, modal);
    initComponents();

    this.module = module;

    this.fieldAddress.getDocument().addDocumentListener(new DocumentListener() {

      @Override
      public void insertUpdate(DocumentEvent e) {
        updateAddress();
      }

      @Override
      public void removeUpdate(DocumentEvent e) {
        updateAddress();
      }

      @Override
      public void changedUpdate(DocumentEvent e) {
        updateAddress();
      }
    });

    this.setLocationRelativeTo(parent);
  }

  private void writeValue() {
    final int address = this.fieldAddress.getIntValue();
    final int value = this.fieldValue.getIntValue();
    this.module.writeMemory(this.module.getCpu(), 0, address, (byte) value);
  }

  private void updateAddress() {
    this.fieldValue.setValue(this.module.readMemory(this.module.getCpu(), 0, this.fieldAddress.getIntValue(), false, false));
  }

  /**
   * This method is called from within the constructor to initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is always
   * regenerated by the Form Editor.
   */
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    jLabel1 = new javax.swing.JLabel();
    jLabel2 = new javax.swing.JLabel();
    fieldAddress = new com.igormaznitsa.zxpoly.tracer.HexValue4Field();
    fieldValue = new com.igormaznitsa.zxpoly.tracer.HexValue2Field();
    buttonClose = new javax.swing.JButton();
    buttonSet = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("Address");
    setResizable(false);

    jLabel1.setText("Address:");

    jLabel2.setText("Value:");

    fieldAddress.setHorizontalAlignment(javax.swing.JTextField.CENTER);

    fieldValue.setHorizontalAlignment(javax.swing.JTextField.CENTER);

    buttonClose.setText("Close");
    buttonClose.addActionListener(this::buttonCloseActionPerformed);

    buttonSet.setText("Set");
    buttonSet.addActionListener(this::buttonSetActionPerformed);

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
    getContentPane().setLayout(layout);
    layout.setHorizontalGroup(
        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jLabel2)
                            .addComponent(jLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(fieldAddress, javax.swing.GroupLayout.DEFAULT_SIZE, 57, Short.MAX_VALUE)
                            .addComponent(fieldValue, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(buttonSet)
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addComponent(buttonClose, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
    );
    layout.setVerticalGroup(
        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(fieldAddress, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(fieldValue, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(buttonSet))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(buttonClose)
                .addContainerGap())
    );

    pack();
  }// </editor-fold>//GEN-END:initComponents

  private void buttonCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonCloseActionPerformed
    this.dispose();
  }//GEN-LAST:event_buttonCloseActionPerformed

  private void buttonSetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonSetActionPerformed
    writeValue();
  }//GEN-LAST:event_buttonSetActionPerformed
  // End of variables declaration//GEN-END:variables
}
